import * as React from 'react';
import * as TextControlBase from '@tesseract-design/web-base-textcontrol';

export interface TextInputProps extends Omit<React.HTMLProps<HTMLInputElement>, 'size' | 'type' | 'style' | 'label' | 'list'> {
  /**
   * Short textual description indicating the nature of the component's value.
   */
  label?: React.ReactNode,
  /**
   * Short textual description as guidelines for valid input values.
   */
  hint?: React.ReactNode,
  /**
   * Size of the component.
   */
  size?: TextControlBase.TextControlSize,
  /**
   * Additional description, usually graphical, indicating the nature of the component's value.
   */
  indicator?: React.ReactNode,
  /**
   * Should the component display a border?
   */
  border?: boolean,
  /**
   * Should the component occupy the whole width of its parent?
   */
  block?: boolean,
  /**
   * Type of the component value.
   */
  type?: TextControlBase.TextControlInputType,
  /**
   * Style of the component.
   */
  style?: TextControlBase.TextControlStyle,
  /**
   * Is the label hidden?
   */
  hiddenLabel?: boolean,
}

/**
 * Component for inputting textual values.
 *
 * This component supports multiline input and adjusts its layout accordingly.
 */
export const TextInput = React.forwardRef<HTMLInputElement, TextInputProps>(
  (
    {
      label = '',
      hint = '',
      indicator = null,
      size = TextControlBase.TextControlSize.MEDIUM,
      border = false,
      block = false,
      type = TextControlBase.TextControlInputType.TEXT,
      style = TextControlBase.TextControlStyle.DEFAULT,
      hiddenLabel = false,
      className: _className,
      placeholder: _placeholder,
      as: _as,
      ...etcProps
    }: TextInputProps,
    ref,
  ) => {
    const textInputBaseArgs: TextControlBase.TextControlBaseArgs = {
      block,
      border,
      size,
      indicator: Boolean(indicator),
      style,
      resizable: false,
      predefinedValues: false,
    };

    return (
      <div
        className={TextControlBase.Root(textInputBaseArgs)}
      >
        <input
          {...etcProps}
          className={TextControlBase.Input(textInputBaseArgs)}
          ref={ref}
          aria-label={label?.toString()}
          type={type}
          data-testid="input"
        />
        {
          border && (
            <span
              data-testid="border"
            />
          )
        }
        {
          label && !hiddenLabel && (
            <div
              data-testid="label"
              className={TextControlBase.LabelWrapper(textInputBaseArgs)}
            >
              {label}
            </div>
          )
        }
        {hint && (
          <div
            className={TextControlBase.HintWrapper(textInputBaseArgs)}
            data-testid="hint"
          >
            <div
              className={TextControlBase.Hint()}
            >
              {hint}
            </div>
          </div>
        )}
        {indicator && (
          <div
            className={TextControlBase.IndicatorWrapper(textInputBaseArgs)}
          >
            {indicator}
          </div>
        )}
      </div>
    );
  }
);

TextInput.displayName = 'TextInput';
